# Ensure user actually wants to run this. This is probably not as fast as it could be, but it gets ran once and isn't painfully slow.
### LOAD PROVIDED DATA ###
import csv
from datetime import datetime
from store.models import *

items = (
    []
)  # A list of items, where each item is a list that contains an id, description, price, and date sold.
with open(
    "example_data/Online Store Project Items for Sale.csv", newline=""
) as csvfile:
    reader = csv.reader(csvfile, delimiter=",", quotechar="|")
    next(reader, None)  # Skip headers
    for row in reader:
        items.append(row)

# Add sale date to items. The CSV is in the same order as the previous.
with open("example_data/Online Store Items Sales.csv", newline="") as csvfile:
    reader = csv.reader(csvfile, delimiter=",", quotechar="|")
    next(reader, None)  # Skip headers
    for index, row in enumerate(reader):
        date = row[
            1
        ]  # Date is stored in the first column, zeroth column is the item ID.
        if date != "":
            date = datetime.strptime(date, "%d/%m/%Y").date()
            items[index].append(
                date
            )  # The first item in the row is the id, we want just the sale date which is in index 1.
        else:  # If an item is unsold it won't have a date.
            items[index].append(None)

item_sets = (
    []
)  # A list of items sets, where each item set is a list that contains the set id, description, quantitity, and then a variable amount of items.
with open(
    "example_data/Online Store Project Sets of Items for Sale.csv", newline=""
) as csvfile:
    reader = csv.reader(csvfile, delimiter=",", quotechar="|")
    next(reader, None)  # Skip headers
    for row in reader:
        item_sets.append(row)

repositories = []  # A list of repositories, where a repository is a list of item ids.
for character in ["A", "B"]:
    with open(
        f"example_data/Online Store Project Items for Sale ALTERNATIVE SHOP {character}.csv",
        newline="",
    ) as csvfile:
        reader = csv.reader(csvfile, delimiter=",", quotechar="|")
        repository = []
        next(reader, None)  # Skip headers
        for row in reader:
            repository.append(
                row[0]
            )  # We only want the ID, and because csv A exists just to mess with me I have to account for an unnecesary column.
        repositories.append(repository)

### SEED DATABASE ###
from store import db, app
from store.models import Item, ItemSet, Repository, User

with app.app_context():
    db.drop_all()  # Start the database anew, will completely remove existing information
    db.create_all()  # Will create all the tables using models.py

    # Add admin for testing
    user = User.create_user(
        username="admin",
        password="password",
        email="admin@website.com",
        phone_number="8383838",
    )
    user.userType = "admin"  # Because of how flasks works, once a row is created it can be further altered via its objects and changes saved on db.session.commit().

    # Generate item sets. This needs to come before items as some of them will be in an item set.
    flask_item_sets = []
    for item_set in item_sets:
        # Items will be added to the sets later.
        flask_item_set = ItemSet(id=item_set[0], description=item_set[1])
        flask_item_sets.append(flask_item_set)

    # Create repositories.
    flask_repositories = []
    for repository in repositories:
        flask_repository = Repository()
        flask_repositories.append(flask_repository)

    # Populate tables.
    for item in items:
        flask_item = Item(
            id=item[0], description=item[1], price=item[2], date_sold=item[3]
        )  # Need to create a flask Item first to add to repositories and item sets, this is what's in the master repository.

        # Initialise repositories.
        for index, repository in enumerate(
            repositories
        ):  # We need the index so we can access the flask object representing the repository in flask_repositories.
            if item[0] in repository:  # Check if item ID is in repo.
                flask_repositories[index].items.append(
                    flask_item
                )  # Add the item to the collection in repository object, creating a many to many relationship.

        # Initialise item set, using mostly the same logic as the prior loop.
        for index, item_set in enumerate(item_sets):
            item_id = item[0]
            item_set_ids = item_set[
                4:7
            ]  # Every element after index 4 is an item in item_set generated from the the csv.
            if item_id in item_set_ids:
                flask_item_sets[index].items.append(flask_item)
        db.session.add(flask_item)

    db.session.commit()  # Apply changes
