from store import app, db
from flask import (
    render_template,
    request,
    flash,
    redirect,
    url_for,
    Flask,
    session,
    abort,
)
from flask_wtf import FlaskForm
from flask_wtf.csrf import generate_csrf
import requests
import datetime
import re
from wtforms import (
    Form,
    StringField,
    validators,
    SelectField,
    TextAreaField,
    ValidationError,
    HiddenField,
)
import json
from store.utility import *
from store.forms import *
import string
import random
import sys


# Official flask-login doc free liecense
# https://flask-login.readthedocs.io/en/latest/
from flask_login import (
    LoginManager,
    UserMixin,
    login_user,
    login_required,
    logout_user,
    current_user,
)

# Official flask-mail docum
# https://pythonhosted.org/Flask-Mail/
from flask_mail import Mail, Message


# Email configuration
app.config["MAIL_SERVER"] = "smtp.gmail.com"
app.config["MAIL_PORT"] = 465
app.config["MAIL_USERNAME"] = "antique.store7111@gmail.com"
app.config["MAIL_PASSWORD"] = "nbouhrtaurcefgvj"
app.config["MAIL_USE_TLS"] = False
app.config["MAIL_USE_SSL"] = True

mail = Mail(app)


# Create an instance of the LoginManager class and initialise
# it with your Flask application to begin using Flask-Login.
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = "login"


@login_manager.user_loader
def load_user(user_id):
    return User.query.get(int(user_id))


def get_user_id():
    if current_user.is_authenticated:
        return current_user.user_id
    else:
        return None


@app.route("/")
@app.route("/index")
def index():
    repositories = Repository.query.all()
    # If the title is unspecified then a default is used that is set in base.html
    return render_template("index.html", title="Home", repositories=repositories)


@app.route("/items")
def items():
    unsold_items = get_unsold_items()

    return render_template("items.html", title="Items", items=unsold_items)


@app.route("/item_sets")
def item_sets():
    items = get_item_sets()
    return render_template("items.html", title="Item Sets", items=items)


@app.route("/item_<int:item_id>")
def item_page(item_id):
    item = get_item_by_id(item_id)
    if not item:
        return "Item not found", 404

    return render_template("item_page.html", item=item)


@app.route("/item_set_<int:item_id>")
def item_set_page(item_id):
    item = get_item_set_by_id(item_id)
    if not item:
        return "Item set not found", 404

    return render_template("item_page.html", item=item)


@app.route("/register", methods=["POST", "GET"])
def register():
    form = RegistrationForm()
    if form.validate_on_submit():
        if User.checkIfUserExist(User, form.username.data):
            error = "The username you chose is already taken."
            return render_template(
                "userContent/register.html", title="Register", error=error
            )

        else:
            user = User.create_user(
                form.username.data,
                form.password.data,
                form.email.data,
                form.phone_number.data,
            )

        return redirect(url_for("index"))
    return render_template("userContent/register.html", form=form)


@app.route("/login", methods=["POST", "GET"])
def login():
    form = LoginForm()
    if form.validate_on_submit():
        username = form.username.data
        password = form.password.data
        user = User.query.filter_by(username=username).first()
        print("hi")
        if user and user.check_password(password):
            print("success")
            login_user(user)
            print(current_user.userType)
            return redirect(url_for("index"))

        else:
            flash("Login unsuccessful. Please check username and password.")
    return render_template("userContent/login.html", form=form)


@app.route("/logout")
@login_required
def logout():
    logout_user()
    return redirect(url_for("index"))


@app.route("/reset_password", methods=["GET", "POST"])
def reset_password():
    if request.method == "POST":
        email = request.form["email"]
        print("the email was :", email)
        user = User.query.filter_by(email=email).first()
        if user:
            verification_code = getcode()
            session["verification_code"] = verification_code
            session["user_id"] = user.user_id
            msg = Message(
                "Password Reset Verification Code",
                sender="your-email@gmail.com",
                recipients=[email],
            )
            msg.body = (
                f"Hello, Your password reset verification code is {verification_code}"
            )
            mail.send(msg)
            flash("A verification code has been sent to your email")
            return redirect(url_for("verify_code"))
        else:
            flash("Invalid email address")
    return render_template("userContent/reset_password.html")


@app.route("/verify_code", methods=["GET", "POST"])
def verify_code():
    if "user_id" not in session or "verification_code" not in session:
        return redirect(url_for("reset_password"))
    if request.method == "POST":
        verification_code = request.form["verification_code"]
        if str(verification_code) == session["verification_code"]:
            session.pop("verification_code", None)
            return redirect(url_for("reset_password_confirm"))
        else:
            flash("Invalid verification code")
    return render_template("userContent/verify_code.html")


@app.route("/reset_password_confirm", methods=["GET", "POST"])
def reset_password_confirm():
    if "user_id" not in session:
        return redirect(url_for("reset_password"))
    user_id = session["user_id"]
    if request.method == "POST":
        password = request.form["password"]
        confirm_password = request.form["confirm_password"]
        if password == confirm_password:
            User.update_passwords(user_id, password)
            session.pop("user_id", None)
            flash("Your password has been successfully reset")
            return redirect(url_for("login"))
        else:
            flash("Passwords do not match")
    return render_template("userContent/reset_password_confirm.html")


def getcode():
    randomchar = [
        str(random.choice(string.ascii_letters)) + str(random.randint(10, 99))
        for _ in range(4)
    ]
    randomdString = ""
    for i in range(0, 4):
        randomdString += randomchar[i % 4]

    return randomdString


@app.route("/reset_security1", methods=["GET", "POST"])
def reset_security1():
    if request.method == "POST":
        email = request.form["email"]
        print("the email was :", email)
        user = User.query.filter_by(email=email).first()
        if user:
            verification_code = getcode()
            session["verification_code"] = verification_code
            session["user_id"] = user.user_id
            msg = Message(
                "security answer, Reset Verification Code",
                sender="your-email@gmail.com",
                recipients=[email],
            )
            msg.body = f"Hello, Your security answer reset verification code is {verification_code}"
            mail.send(msg)
            flash("A verification code has been sent to your email")
            return redirect(url_for("verify_code_security1"))
        else:
            flash("Invalid email address")
    return render_template("userContent/reset_security1.html")


@app.route("/verify_code_security1", methods=["GET", "POST"])
def verify_code_security1():
    if "user_id" not in session or "verification_code" not in session:
        return redirect(url_for("verify_code_security1"))
    if request.method == "POST":
        verification_code = request.form["verification_code"]
        if str(verification_code) == session["verification_code"]:
            session.pop("verification_code", None)
            return redirect(url_for("verify_code_security1_confirm"))
        else:
            flash("Invalid verification code")
    return render_template("userContent/verify_code_security1.html")


@app.route("/account")
@login_required
def account():
    if current_user.userType == "standard":
        return redirect(url_for("standardUser"))
    elif current_user.userType == "admin":
        return redirect(url_for("admin"))
    else:
        flash("Unauthorized access please contact us on antique.store7111@gmail.com")
        return redirect(url_for("index"))


@app.route("/standardUser")
@login_required
def standardUser():
    if current_user.userType != "standard":
        flash("Unauthorized access no admins allowed please create a standard account")
        return redirect(url_for("home"))
    else:
        return render_template(
            "userContent/user.html", user=current_user, userType=current_user.userType
        )


@app.route("/admin")
@login_required
def admin():
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))
    else:
        return render_template(
            "userContent/user.html", user=current_user, userType=current_user.userType
        )


@app.route("/database_management", methods=["GET", "POST"])
@login_required
def database_management():
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))
    # No need for an else statement as early return will
    # prevent access anyway.
    access_data_form = AccessDataForm()
    if access_data_form.validate_on_submit():
        if access_data_form.table.data == "Item":
            table = Item
        elif access_data_form.table.data == "Item Set":
            table = ItemSet
        id = access_data_form.id.data
        item = table.query.get(id)
        if item == None:
            flash("Invalid ID!")

        # No point to using another page, just use the same template.
        return render_template(
            "userContent/database_management.html",
            access_data_form=access_data_form,
            main_item=item,
            is_item=type(item) is Item,
        )

    return render_template(
        "userContent/database_management.html",
        access_data_form=access_data_form,
        main_item=None,
    )


@app.route("/database_management_add", methods=["GET", "POST"])
@login_required
def database_management_add():
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))

    form = AddForm()
    if form.validate_on_submit():
        description = form.description.data
        price = form.price.data

        item = None
        if form.table.data == "Item":
            item = Item(description=description, price=price)
        elif form.table.data == "Item Set":
            item = ItemSet(description=description)
        db.session.add(item)
        db.session.commit()

        access_data_form = AccessDataForm()
        return render_template(
            "userContent/database_management.html",
            access_data_form=access_data_form,
            main_item=item,
            is_item=type(item) is Item,
        )

    return render_template(
        "userContent/database_management_add.html",
        form=form,
    )


@app.route("/add_item_to_set", methods=["GET", "POST"])
def add_item_to_set():
    print("Adding Item", file=sys.stderr)
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))

    item_id = request.form.get("item_id")
    set_id = request.form.get("set_id")
    item_set = ItemSet.query.get(set_id)
    item = Item.query.get(item_id)
    print("Adding Item:", item_id, "To Set:", set_id, file=sys.stderr)

    if item_set == None or item == None:
        abort(406)
    if item in item_set.items:
        abort(406)

    print("Adding Item:", item_id, "To Set:", set_id, file=sys.stderr)
    item_set.items.append(item)
    db.session.commit()
    return "ok"


@app.route("/delete_item", methods=["DELETE"])
def delete_item():
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))
    id = request.form.get("item_id")
    item = Item.query.get(id)
    if item == None:
        print("Could not delete item:", id, file=sys.stderr)
        abort(406)
    print("Deleting Item:", id, file=sys.stderr)
    remove_item(item)
    return "ok"


@app.route("/delete_item_set", methods=["DELETE"])
def delete_item_set():
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))
    id = request.form.get("item_id")
    item = ItemSet.query.get(id)
    if item == None:
        abort(406)
    print("Deleting Item Set:", id, file=sys.stderr)
    remove_item_set(item)
    return "ok"


@app.route("/delete_item_from_set", methods=["DELETE"])
def delete_item_from_set():
    if current_user.userType != "admin":
        flash("Unauthorized access")
        return redirect(url_for("home"))
    item_id = request.form.get("item_id")
    set_id = request.form.get("set_id")
    item_set = ItemSet.query.get(set_id)
    item = Item.query.get(item_id)

    if item_set == None or item == None:
        abort(406)

    print("Deleting Item:", item_id, "From Set:", set_id, file=sys.stderr)
    item_set.items.remove(item)
    db.session.commit()
    return "ok"


@app.route("/ChangeUsername/", methods=["GET", "POST"])
@login_required
def ChangeUsername():
    if request.method == "POST":
        user_id = current_user.user_id
        password = request.form["password"]
        NewUsername = request.form["NewUsername"]
        user = User.query.get(user_id)
        if user and user.check_password(password):
            User.update_username(user_id, NewUsername)
            flash("username successfully changed.")
        else:
            flash("Wrong password.")

    return render_template("userContent/ChangeUsername.html")


@app.route("/ChangeEmail/", methods=["GET", "POST"])
@login_required
def ChangeEmail():
    if request.method == "POST":
        user_id = current_user.user_id
        password = request.form["password"]

        NewEmail = request.form["NewEmail"]
        user = User.query.get(user_id)
        if user and user.check_password(password):
            User.update_email(user_id, NewEmail)
        else:
            flash("Wrong password.")

    return render_template("userContent/ChangeEmail.html")


@app.route("/ChangePhNumber/", methods=["GET", "POST"])
@login_required
def ChangePhNumber():
    if request.method == "POST":
        user_id = current_user.user_id
        password = request.form["password"]
        NewNumber = request.form["NewNumber"]
        user = User.query.get(user_id)
        if user and user.check_password(password):
            User.update_Number(user_id, NewNumber)
            flash("New Number successfully changed.")
        else:
            flash("Wrong password.")

    return render_template("userContent/ChangePhNumber.html")


def validate_cvv_2(form, field):
    regex = "^[0-9]{3,4}$"
    p = re.compile(regex)
    if not re.search(p, field.data):
        raise ValidationError("CVV2 is incorrect")


def validate_address(form, field):
    # Uses the google maps geocoding api to check if a postcode or address exists

    url = "https://maps.googleapis.com/maps/api/geocode/json"

    api_key = "AIzaSyCz4xyIl5yaIg869orL07ye2ainw7kv5Pc"

    params = {"address": field, "key": api_key}

    response = requests.get(url, params=params).json()

    # Check if response status is "OK" and has at least one result
    if response["status"] == "OK" and len(response["results"]) > 0:
        print("Valid address or postcode")
        pass
    else:
        raise ValidationError("Enter valid address")


class CheckoutForm(Form):
    months = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]
    today = datetime.date.today()
    year = int(today.year)
    years = [year]
    for year in range(year, year + 50, 1):
        years.append(year)
    name = StringField(
        "Name on card", [validators.Length(min=2, max=30), validators.DataRequired()]
    )
    card_number = StringField(
        "Card Number", [validators.Length(min=15, max=16), validators.DataRequired()]
    )
    exp_date_month = SelectField("Month", [validators.DataRequired()], choices=months)
    exp_date_year = SelectField("Year", [validators.DataRequired()], choices=years)
    cvv_2 = StringField(
        "CVV2",
        [validators.Length(min=3, max=4), validators.DataRequired(), validate_cvv_2],
    )
    address = TextAreaField(
        "Address",
        [
            validators.Length(min=10, max=250),
            validators.DataRequired(),
            validate_address,
        ],
    )
    postcode = StringField(
        "Postcode",
        [
            validators.Length(min=6, max=10),
            validators.DataRequired(),
            validate_address,
        ],
    )
    # Cross-site request forgery token to make sure requests are sent from our site
    csrf_token = HiddenField()


@app.route("/order_summary")
def order_summary():
    total_price = 0
    items = []
    if "basket" in session:
        address = session.get("address", "")
        for item_id, item in session["basket"].items():
            items.append(item)
            total_price = total_price + item["price"]
        session.clear()

        return render_template(
            "order_summary.html", items=items, address=address, total_price=total_price
        )
    else:
        return redirect(url_for("index"))


@app.route("/checkout", methods=["POST", "GET"])
def checkout():
    items = []
    total_price = 0
    form = CheckoutForm(request.form)
    # Get items stored in basket and add to list
    for item_id, item in session["basket"].items():
        items.append(item)
        total_price = total_price + item["price"]
    if request.method == "POST" and form.validate():
        session["address"] = str(form.address.data + ", " + form.postcode.data)
        for item_id in session["basket"]:
            item = session["basket"][item_id]
            if item["is_item_set"] == True:
                print("Sell item set: ", item_id)
                sell_item_set(get_item_set_by_id(item_id))
            else:
                print("Sell item: ", item_id)
                sell_item(get_item_by_id(item_id))
        return redirect(url_for("order_summary"))
    csrf_token = generate_csrf()
    return render_template(
        "checkout.html",
        form=form,
        total_price=total_price,
        csrf_token=csrf_token,
        items=items,
    )


@app.route("/basket")
def basket():
    items = []
    total_price = 0
    if "basket" not in session:
        session["basket"] = {}
    else:
        for item_id, item in session["basket"].items():
            items.append(item)
            total_price = total_price + item["price"]

    return render_template(
        "basket.html", title="Basket", items=items, total_price=total_price
    )


@app.route("/add_to_basket", methods=["POST"])
def add_to_basket():
    item_id = request.form["item_id"]

    item_obj = get_item_by_id(item_id)

    item_dict = {
        "id": item_obj.id,
        "description": item_obj.description,
        "price": item_obj.price,
        "is_item_set": False,
    }

    if "basket" not in session:
        session["basket"] = {}
    item_sets = get_itemsets_by_item_id(item_id)
    is_in_basket = False
    for item_set in item_sets:
        if item_set.id in session["basket"]:
            is_in_basket = True
            flash("Item already in basket in set")

    if item_id in session["basket"]:
        flash("Item already in basket")
    elif not is_in_basket:
        session["basket"][item_id] = item_dict
        flash("Item added to basket: " + item_obj.description)

    return redirect(url_for("item_page", item_id=item_id))


@app.route("/add_to_basket_set", methods=["POST"])
def add_to_basket_set():
    item_id = request.form["item_id"]
    item_obj = get_item_set_by_id(item_id)

    item_dict = {
        "id": item_obj.id,
        "description": item_obj.description,
        "price": item_obj.price,
        "is_item_set": True,
    }
    if "basket" not in session:
        session["basket"] = {}
    is_in_basket = False
    for item in item_obj.items:
        if item.id in session["basket"]:
            is_in_basket = True
            flash("Item already in basket in item set.")
    if item_id in session["basket"]:
        flash("Item already in basket")
    elif not is_in_basket:
        session["basket"][item_id] = item_dict
        flash("Item added to basket: " + item_obj.description)
        print(session["basket"])
    return redirect(url_for("item_set_page", item_id=item_id))


@app.route("/remove_item", methods=["POST"])
def remove_item_basket():  # Calling this remove_item shadows the utility function remove_item()
    if request.method == "POST":
        item_id = request.form["item"]

        if item_id in session["basket"]:
            print("here")
            del session["basket"][item_id]
            session.modified = True

    return redirect(url_for("basket"))


@app.route("/addAdress", methods=["GET", "POST"])
@login_required
def addAdress():
    if request.method == "POST":
        user_id = current_user.user_id
        country = request.form["country"]
        homeNumber = request.form["homeNumber"]
        streetName = request.form["streetName"]
        city = request.form["city"]
        phoneNumber = request.form["phoneNumber"]
        contactEmail = request.form["contactEmail"]

        # user = User.query.get(user_id)

        newAdress = Adresses.create_Adresses(
            user_id=user_id,
            country=country,
            homeNumber=homeNumber,
            streetName=streetName,
            city=city,
            phoneNumber=phoneNumber,
            contactEmail=contactEmail,
        )
        flash("Your address has been added!")
        return redirect(url_for("addAdress"))
    return render_template("userContent/address.html")


@app.route("/view_address", methods=["GET"])
@login_required
def view_address():
    user_id = current_user.user_id
    addresses = Adresses.query.filter_by(user_id=user_id).all()

    if not addresses:
        flash("You don't have any saved addresses. you can add here")
        return redirect(url_for("addAdress", user_id=user_id))
    else:
        return render_template("userContent/view_address.html", addresses=addresses)


@app.context_processor
def base():
    """So that search works on every page, and so that the search form does not need to be passed
    to every template a context processor is used to inject a variable (in this case search_form)
    into every template.
    """
    form = SearchForm()
    return dict(search_form=form)


@app.route("/search", methods=["POST"])
def search():
    form = SearchForm()
    if form.validate_on_submit():
        items = get_by_keyword(form.query.data)
        return render_template("items.html", items=items)
