from store import app

""" An instance of the database needs to exist to unit test,
as the database is seeded from file this can be used as a test database.
Really there should be an inmemory database for testing purposes, but
further research into how flask works with such a thing needs to be done.
"""

from store.utility import *
import unittest


class TestUtility(unittest.TestCase):
    def test_sold_items_contains_sold(self):
        sold_items = get_sold_items()
        # Should return true, 12401 is sold.
        contains_sold = any(item.id == 12401 for item in sold_items)
        self.assertEqual(contains_sold == True, True)

    def test_sold_items_contains_unsold(self):
        sold_items = get_sold_items()
        # Should return false, 12414 hasn't been sold.
        contains_unsold = any(item.id == 12414 for item in sold_items)
        self.assertEqual(contains_unsold == False, True)

    def test_unsold_items_contains_sold(self):
        unsold_items = get_unsold_items()
        # Should return false, 12401 is sold.
        contains_sold = any(item.id == 12401 for item in unsold_items)
        self.assertEqual(contains_sold == False, True)

    def test_unsold_items_contains_unsold(self):
        unsold_items = get_unsold_items()
        # Should return true, 12414 hasn't been sold.
        contains_unsold = any(item.id == 12414 for item in unsold_items)
        self.assertEqual(contains_unsold == True, True)

    def test_get_itemsets_by_item_id(self):
        itemsets = get_itemsets_by_item_id(12542)
        contains_itemset = any(itemset.id == 25701 for itemset in itemsets)
        contains_itemset2 = any(itemset.id == 25708 for itemset in itemsets)
        self.assertEqual(contains_itemset == True, True)
        self.assertEqual(contains_itemset2 == True, True)

    def test_sell_item(self):
        items = get_unsold_items()
        item = items[0]
        sell_item(item)
        items = get_unsold_items()
        contains_sold = any(item.id == 12414 for item in items)
        self.assertEqual(contains_sold == False, True)

    def test_get_by_keyword(self):
        results = get_by_keyword("vase")
        self.assertEqual(len(results) > 0, True)

    def test_get_similar_items(self):
        results = get_similar_items("Chinese Porcelaine Vases set")
        is_search_too_wide = any(
            "chinese" not in descriptable.description.lower()
            for descriptable in results
        )

        self.assertEqual(len(results) > 0 and not is_search_too_wide, True)

    def add_item_to_database(self):
        item = ["Vase with golden trimming", 12]
        add_item(item[0], item[1])
        added_item = Item.query.filter_by(description="Vase with golden trimming").all()
        self.assertEqual(item[0], added_item.description)

    def test_update_set(self):
        """This should remove item 12517 from two sets, 25701, 25708, and add 12568 to those two sets.
        It does this by interpreting its description Gold Sovereign Coin 1972.
        """
        item = get_item_by_id(12517)
        update_set(item)
        item_sets = get_itemsets_by_item_id(12568)
        # With the expected example there should only be 2 retrieved sets.
        self.assertEqual(len(item_sets), 2)
        # This item should now be in the item set.
        expected_item = get_item_by_id(12568)
        for item_set in item_sets:
            self.assertIn(expected_item, item_set.items)


if __name__ == "__main__":
    with app.app_context():
        unittest.main()
